% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sbs.search.R
\name{sbs.search}
\alias{sbs.search}
\title{Sequential backward search method}
\usage{
sbs.search(data, y, zfaResult, test, fuzzy = FALSE)
}
\arguments{
\item{data}{a data frame or numeric matrix. Genotypes should be coded as 0,1 or 2.}

\item{y}{a numeric vector with two levels. Phenotype values are coded as 0 or 1.}

\item{zfaResult}{results of ZFA optimization, the output of "zfa" and "fuzzy.zfa" functions.}

\item{test}{a character to choose the rare variant method that combines with the Zoom- Focus Algorithm.If test = "SKAT", the SKAT of variance component test is applied. If test = "SKATO", the SKAT-O of combination method test is applied. If test = "burden", the weighted burden test is applied. If test = "wtest", the W-test of burden test category is applied.}

\item{fuzzy}{a logical value indicating whether zfaResult is the result of "fuzzy.zfa" function. Default = FALSE.}
}
\value{
The function returns a list with two components:

\item{sbs.result}{a data frame of results aggregated over the sequential backward search process.}

\item{sbs.best}{a list including the index of removed and selected variants, and the p-value of the selected variants. Note that the p-value has already been adjusted by Bonferroni correction.}
}
\description{
This function sequentially eliminates one variant within the optimal region located by ZFA at each iteration until obtaining a subset of variants that are enriched for association with the phenotype.
}
\details{
This function further refines the optimal region located by ZFA.
}
\examples{

data(zfa.example) 
attach(zfa.example)

## fast-zoom with wtest, all zoomed regions passed to focusing step, 
## and output all results
zfa.result<-zfa(X,y,bin = 32,fast.path = TRUE, 
            filter.pval=NULL,output.pval=NULL,test = "wtest")

## further refines the optimal region located by ZFA.
zfa.sbs <- sbs.search(X,y,zfa.result,"wtest",fuzzy = FALSE)

}
