#include <RcppArmadillo.h>
// [[Rcpp::depends(RcppArmadillo)]]
using namespace Rcpp;
using namespace arma;
#include <cmath>
#include <omp.h>

// [[Rcpp::export]]
double wtest(arma::mat a,arma::rowvec& b){
  int cols = a.n_cols;
  int rows = a.n_rows;
  mat all_table=zeros(6,cols);
  for(int i=0;i<cols;i++){
    int index=0;
    for(int j=0;j<rows;j++){
      index = int(a(j,i)) + b[j]*3;
      all_table(index,i) +=1;
    }
  }
  mat sum_table = sum(all_table,1);
  sum_table.reshape(3,2);
  colvec T = sum(sum_table,1);
  mat cont_table=sum_table.rows(find(T!=0));
  if(cont_table.min()==0){
    cont_table+=0.5;
  }
  mat hf={{0.5,1},{0.667,2}};
  // x2
  int df = cont_table.n_rows-2;//下标从0开始
  mat o = cont_table;
  rowvec colSum = sum(cont_table,0);
  for(unsigned int i =0;i<cont_table.n_cols;i++)
    cont_table.col(i) /= colSum(i);
  mat p = cont_table/(1-cont_table);
  vec OR = p.col(1)/p.col(0);
  rowvec s= sum(o,0);
  o.resize(o.n_rows,4);
  o.col(2)=s(0)-o.col(0);
  o.col(3)=s(1)-o.col(1);
  vec sd=sqrt(sum((1/o),1));
  double x2_value =sum(square(log(OR)/sd));
  double w_value = x2_value*hf(df,0);
  double p_value = R::pchisq(w_value,hf(df,1),0,0);//yxzhang
  return p_value;
}

// [[Rcpp::export]]
double wtest_zfa(arma::mat& a,arma::rowvec& b,int lower_idx,int upper_idx){
  // int cols = a.n_cols;
  int rows = a.n_rows;
  mat all_table=zeros(6,(upper_idx-lower_idx+1));
  for(int i=lower_idx,idx=0;i<=upper_idx;i++,idx++){
    int index=0;
    for(int j=0;j<rows;j++){
      index = int(a(j,i)) + b(j)*3;
      all_table(index,idx) +=1;
    }
  }
  mat sum_table = sum(all_table,1);
  sum_table.reshape(3,2);
  colvec T = sum(sum_table,1);
  mat cont_table=sum_table.rows(find(T!=0));
  if(cont_table.min()==0){
    cont_table+=0.5;
  }
  mat hf={{0.5,1},{0.667,2}};
  // x2
  int df = cont_table.n_rows-2;//下标从0开始
  mat o = cont_table;
  rowvec colSum = sum(cont_table,0);
  for(unsigned int i =0;i<cont_table.n_cols;i++)
    cont_table.col(i) /= colSum(i);
  mat p = cont_table/(1-cont_table);
  vec OR = p.col(1)/p.col(0);
  rowvec s= sum(o,0);
  o.resize(o.n_rows,4);
  o.col(2)=s(0)-o.col(0);
  o.col(3)=s(1)-o.col(1);
  vec sd=sqrt(sum((1/o),1));
  double x2_value =sum(square(log(OR)/sd));
  double w_value = x2_value*hf(df,0);
  double p_value = R::pchisq(w_value,hf(df,1),0,0);//yxzhang
  return p_value;
}

// 最佳区间index已经加1
// [[Rcpp::export]]
arma::mat zoom(arma::mat& data, arma::rowvec& y,arma::mat& bound_index,int idx){
  int P=bound_index(idx,1)-bound_index(idx,0)+1;
  int R=std::floor(std::log2(P));//二分次数
  std::vector<int> r(R) ; // vector with R ints.
  std::iota (std::begin(r), std::end(r), 1); // Fill with 1, ..., R.
  std::vector<int> nr(r.size());
  std::vector<double> d(r.size());
  for(int i=0;i<(int)r.size();i++){
    nr[i]=pow(2,r[i]-1);
    d[i]=(double)P/nr[i];
  }
  int sum = std::accumulate(nr.begin(), nr.end(), 0);
  arma::mat result(sum,3);//result
  int lower_all=0;
  int upper_all=0;
  double pvalue=1;
  int count=0;
  for(int j=0;j<R;j++){//计算每个区间的pvalue，可以考了继续采用bound_index(fuzzy zfa)
    double d1=d[j];
    int nr1=nr[j];
    std::vector<int> c(nr1) ; // vector with nr1 ints.
    std::iota (std::begin(c), std::end(c), 1); // Fill with 1, ..., nr1.
    for(int k=0;k<nr1;k++){
      lower_all=(int)(bound_index(idx,0)+d1*(c[k]-1)-1);
      upper_all=(int)(bound_index(idx,0)+d1*c[k]-2);
      pvalue=wtest_zfa(data,y,lower_all,upper_all)*nr1;
      pvalue=(pvalue>1)?1:pvalue;
      lower_all+=1;
      upper_all+=1;
      result.row(count)=arma::rowvec({(double)(lower_all), (double)(upper_all),pvalue});
      //对于fuzzy方法可以根据upper_all-lower_all>3可以再分一次
      count++;
    }
  }
  return(result.row(result.col(2).index_min()));
}

// 返回最佳区间index已经加1
// [[Rcpp::export]]
arma::rowvec zoom_fast(arma::mat& data, arma::rowvec& y,arma::mat& bound_index,int idx){
  int cols=bound_index(idx,1)-bound_index(idx,0)+1;
  int lower=bound_index(idx,0)-1;
  int upper=bound_index(idx,1)-1;
  int mid=cols-1;
  int len=std::floor(log2(cols));
  std::vector<int> lower_all(len);
  std::vector<int> upper_all(len);
  std::vector<double> p_val(len);
  std::vector<double> p_val_left_right(2);
  p_val[0]=wtest_zfa(data,y,lower,upper);
  lower_all[0]=lower;
  upper_all[0]=upper;
  for(int idx=1;idx<len;idx++){
    mid=mid/2;
    p_val_left_right[0]=wtest_zfa(data,y,lower,lower+mid);
    p_val_left_right[1]=wtest_zfa(data,y,upper-mid,upper);
    if(p_val_left_right[0]<=p_val_left_right[1]){
      upper=upper-mid-1;
      p_val[idx]=p_val_left_right[0];
    }else{
      lower=lower+mid+1;
      p_val[idx]=p_val_left_right[1];
    }
    lower_all[idx]=lower;
    upper_all[idx]=upper;
  }
  int min_p_val_idx = std::min_element(p_val.begin(),p_val.end()) - p_val.begin();
  double p_val_corr=(p_val[min_p_val_idx]>1)?1:p_val[min_p_val_idx];
  lower_all[min_p_val_idx]+=1;
  upper_all[min_p_val_idx]+=1;
  arma::rowvec result_final ={(double)lower_all[min_p_val_idx], (double)upper_all[min_p_val_idx], p_val_corr};
  return(result_final);
}

// [[Rcpp::export]]
arma::rowvec focus(arma::mat& data,arma::rowvec& y,arma::mat& bound_index,int idx,arma::rowvec ZoomResult,int bin,bool fast_path,double filter_pval,bool adjacent){
  filter_pval=(filter_pval>1)?1:filter_pval;
  // std::vector<double> result_final(4);
  arma::rowvec result_final(5);
  int d=(int)(ZoomResult[1]-ZoomResult[0]+1);
  int inward;
  int outward;
  double steps=0.0;//所有计算次数，用于SBS p value校验
  // adjacent
  int low_idx = adjacent?0:(bound_index(idx,0)-1);
  int up_idx = adjacent?data.n_cols:bound_index(idx,1);
  if(ZoomResult[2]<=filter_pval){
    if(d==2){
      inward=0;
      outward=2*d;
    }else{
      inward=std::floor(d/4);
      outward=std::floor(d/2);
    }
    std::vector<double> p_val(inward+outward+1,1.0);
    std::vector<int> idx(inward+outward+1,-1);
    int count=0;
    for(int i=(-outward);i<=inward;i++){
      if((ZoomResult[0]+i-1)>=low_idx){// 如果不选择搜索临近region,传入bound index：(arma::mat& bound_index,int idx),>=bound_index(idx,0)-1
        p_val[count]=wtest_zfa(data,y,(int)(ZoomResult[0]+i-1),(int)(ZoomResult[1]-1));
        idx[count]=(int)(ZoomResult[0]+i-1);
        count++;
      }
    }
    int min_idx = std::min_element(p_val.begin(),p_val.end()) - p_val.begin();
    std::vector<double> p_val2(inward+outward+1,1.0);
    std::vector<int> idx2(inward+outward+1,-1);
    int count2=0;
    // int cols=data.n_cols;
    for(int i=(-inward);i<=outward;i++){
      if((ZoomResult[1]+i-1)< up_idx){// 如果不选择搜索临近region,传入bound index：(arma::mat& bound_index,int idx),<=bound_index(idx,1)-1
        p_val2[count2]=wtest_zfa(data,y,idx[min_idx],(int)(ZoomResult[1]+i-1));
        idx2[count2]=(int)(ZoomResult[1]+i-1);
        count2++;
      }
    }
    int min_idx2 = std::min_element(p_val2.begin(),p_val2.end()) - p_val2.begin();
    int k=count + count2-2;
    double pvalue_right_corrected;
    if(fast_path){
      pvalue_right_corrected=p_val2[min_idx2]*(bin/2.0+std::log2(bin)-1+k);//思考将用于pval校验的数值传入SBS，用于SBS的pval校验
      steps=(bin/2.0+std::log2(bin)-1+k);
    }else{
      pvalue_right_corrected=p_val2[min_idx2]*(bin-1+k);//思考将用于pval校验的数值传入SBS，用于SBS的pval校验
      steps=(bin-1+k);
    }
    pvalue_right_corrected=(pvalue_right_corrected>1)?1:pvalue_right_corrected;
    double lower_bound=idx[min_idx]+1;
    double upper_bound=idx2[min_idx2]+1;
    double region_size=idx2[min_idx2]-idx[min_idx]+1;
    result_final ={lower_bound, upper_bound, region_size,pvalue_right_corrected,steps};
    // result_final=((double)(idx[min_idx]+1),(double)(idx2[min_idx2]+1),(double)(idx2[min_idx2]-idx[min_idx]+1),pvalue_right_corrected,steps);
  } else{
    if(fast_path){
      ZoomResult[2]=ZoomResult[2]*(bin/2.0+std::log2(bin)-1);
      ZoomResult[2]=(ZoomResult[2]>1)?1:ZoomResult[2];
      steps=(bin/2.0+std::log2(bin)-1);
    }else{
      ZoomResult[2]=ZoomResult[2]/(bin/(ZoomResult[1]-ZoomResult[0]+1))*(bin-1);
      ZoomResult[2]=(ZoomResult[2]>1)?1:ZoomResult[2];
      steps=(bin/(ZoomResult[1]-ZoomResult[0]+1))*(bin-1);
    }
    double region_size=ZoomResult[1]-ZoomResult[0]+1;
    result_final ={ZoomResult[0], ZoomResult[1], region_size,ZoomResult[2],steps};//fast zoom index已经加1
  }
  return(result_final);
}

// [[Rcpp::export]]
arma::mat zfa_wtest(arma::mat& bound_index,arma::mat& data, arma::rowvec& y, int bin,bool fast_path,double filter_pval,bool adjacent=1,int thd_num=1){
  filter_pval=(filter_pval>1)?1:filter_pval;
  int splits=bound_index.n_rows;
  arma::mat result_final(splits,5);
  if(fast_path){
#pragma omp parallel for num_threads(thd_num)
    for(int i=0;i<splits;i++){
      arma::rowvec zoom_result=zoom_fast(data,y,bound_index,i);
      result_final.row(i)=focus(data,y,bound_index,i,zoom_result,bin,fast_path,filter_pval,adjacent);
    }
  }else{
#pragma omp parallel for num_threads(thd_num)
    for(int i=0;i<splits;i++){
      arma::rowvec zoom_result=zoom(data,y,bound_index, i);
      result_final.row(i)=focus(data,y,bound_index,i,zoom_result,bin,fast_path,filter_pval,adjacent);
    }
  }
  return(result_final);
}


// [[Rcpp::export]]
SEXP sbs(arma::mat data,arma::rowvec& y,SEXP objs,SEXP tests,arma::uvec zfa_idxs){
  Rcpp::List obj(objs);
  std::string test=Rcpp::as<std::string>(tests);
  Rcpp::Environment env=Environment::namespace_env("SKAT");
  Rcpp::Function func=env["SKATBinary"];
  Rcpp::Environment utils("package:utils");
  Rcpp::Function combn = utils["combn"];
  Rcpp::Function setdiff("setdiff");
  int idx_len=zfa_idxs.n_elem;
  arma::umat combs = Rcpp::as<arma::umat>(combn(zfa_idxs,idx_len-1));
  arma::mat result_final(idx_len,2);
  double p_val_all=1.0;
  if(test=="wtest"){
    p_val_all=wtest(data.cols(zfa_idxs-1),y);
  }else{
    // arma::mat data_temp=data.cols(zfa_idxs-1);
    p_val_all=Rcpp::as<List>(func((arma::mat)(data.cols(zfa_idxs-1)),obj,Named("method",test)))["p.value"];
    // p_val_all=Rcpp::as<List>(func(data_temp,obj,Named("method",test)))["p.value"];
  }
  // Rprintf("p_val_all:%f",p_val_all);
  result_final.row(0)=arma::rowvec({0,p_val_all});
  int count=1;
  arma::uvec zfa_idxs_temp=zfa_idxs;
  while(idx_len>1){
    // Rprintf("len:%d\n",idx_len);
    std::vector<double> p_val(combs.n_cols,1.0);
    Rcpp::NumericVector p_val2(combs.n_cols);
    for(int i=0;i<int(combs.n_cols);i++){
      // wtest,保存每一个p-value
      if(test=="wtest"){
        p_val[i]=wtest(data.cols(combs.col(i)-1),y);
      }else{
        arma::mat data_temp = data.cols(combs.col(i)-1);
        p_val[i]=Rcpp::as<List>(func(data_temp,obj,Named("method",test)))["p.value"];
      }
      
      // Rprintf("p_val_all:%f",p_val[i]);
    }
    int min_idx = std::min_element(p_val.begin(),p_val.end()) - p_val.begin();
    double del_idx = Rcpp::as<double> (setdiff(zfa_idxs_temp,combs.col(min_idx)));
    result_final.row(count)=arma::rowvec({del_idx,p_val[min_idx]});
    zfa_idxs_temp=arma::uvec(combs.col(min_idx));
    idx_len--;
    combs = Rcpp::as<arma::umat>(combn(combs.col(min_idx),idx_len-1));
    count++;
  }
  return Rcpp::wrap(result_final);
  // p_val*(n*(n+1)/2-1)
  int min_idx=result_final.col(1).index_min();
  if(result_final(min_idx,1)>p_val_all){
    return Rcpp::List::create(Rcpp::Named("idxs")=zfa_idxs,
                              Rcpp::Named("pval")=p_val_all);
  }else{
    return Rcpp::List::create(Rcpp::Named("idxs")=setdiff(zfa_idxs,result_final(span(0, min_idx), 0)),
                              Rcpp::Named("pval")=result_final(min_idx,1));
  }
}
// sbs返回p_val后再乘以zfa返回的steps+(n*(n+1)/2-1)，后者是sbs总搜索步数
// obj <- obj<-SKAT::SKAT_Null_Model(zfa.example$y ~ 1, out_type="D")
// a <- sbs(zfa::zfa.example$X,zfa::zfa.example$y,obj,"SKAT",c(20:25))
// setdiff(c(20:25),a[2:which.min(a[,2]),1])

// zfa_wtest(bound_index,allData[[6]],y,32,FALSE,1,1,5)
